#import <Foundation/Foundation.h>
#import <Cocoa/Cocoa.h>
#import <Metal/Metal.h>
#import <MetalKit/MetalKit.h>
#import <QuartzCore/QuartzCore.h>
#include <iostream>
#include <thread>
#include <chrono>
    using namespace std::chrono_literals;
@interface MyAppDelegate : NSObject <NSApplicationDelegate> {
@public
    BOOL seenFirstActivate;
}

- (id)init;
- (void)localeDidChange:(NSNotification *)notification;
@end

@implementation MyAppDelegate : NSObject
- (id)init
{
    self = [super init];
    if (self) {
        NSNotificationCenter *center = [NSNotificationCenter defaultCenter];

        seenFirstActivate = NO;

        [center addObserver:self
                   selector:@selector(windowWillClose:)
                       name:NSWindowWillCloseNotification
                     object:nil];

        [center addObserver:self
                   selector:@selector(focusSomeWindow:)
                       name:NSApplicationDidBecomeActiveNotification
                     object:nil];

        [center addObserver:self
                   selector:@selector(localeDidChange:)
                       name:NSCurrentLocaleDidChangeNotification
                     object:nil];
    }

    return self;
}

- (void)dealloc
{
    NSNotificationCenter *center = [NSNotificationCenter defaultCenter];
    [center removeObserver:self name:NSWindowWillCloseNotification object:nil];
    [center removeObserver:self name:NSApplicationDidBecomeActiveNotification object:nil];
    [center removeObserver:self name:NSCurrentLocaleDidChangeNotification object:nil];
    [super dealloc];
}

- (void)windowWillClose:(NSNotification *)notification;
{
}

- (void)focusSomeWindow:(NSNotification *)aNotification
{
    if (!seenFirstActivate) {
        seenFirstActivate = YES;
        return;
    }
}

- (void)localeDidChange:(NSNotification *)notification;
{
}

- (BOOL)application:(NSApplication *)theApplication openFile:(NSString *)filename
{
}

- (void)applicationDidFinishLaunching:(NSNotification *)notification
{
        for (NSRunningApplication *i in [NSRunningApplication runningApplicationsWithBundleIdentifier:@"com.apple.dock"]) {
            [i activateWithOptions:NSApplicationActivateIgnoringOtherApps];
            break;
        }
        [NSApp activateIgnoringOtherApps:YES];
}

- (void)handleURLEvent:(NSAppleEventDescriptor *)event withReplyEvent:(NSAppleEventDescriptor *)replyEvent
{
}

@end

int main(){
    
    [NSAutoreleasePool new];
    [NSApplication sharedApplication];
    [NSApp setActivationPolicy:NSApplicationActivationPolicyRegular];
   
    [NSApp finishLaunching];
    NSDictionary *appDefaults = [[NSDictionary alloc] initWithObjectsAndKeys:
                                 [NSNumber numberWithBool:NO], @"AppleMomentumScrollSupported",
                                 [NSNumber numberWithBool:NO], @"ApplePressAndHoldEnabled",
                                 [NSNumber numberWithBool:YES], @"ApplePersistenceIgnoreState",
                                 nil];
    [[NSUserDefaults standardUserDefaults] registerDefaults:appDefaults];
    [appDefaults release];
    static MyAppDelegate *appDelegate = [[MyAppDelegate alloc] init];
    [(NSApplication *)NSApp setDelegate:appDelegate];

    
    NSWindow *window = [[[NSWindow alloc] initWithContentRect:NSMakeRect(0, 0, 350, 350)
        styleMask:NSWindowStyleMaskTitled|NSWindowStyleMaskClosable|NSWindowStyleMaskResizable
        backing:NSBackingStoreBuffered defer:NO]
        autorelease];
              
    [window cascadeTopLeftFromPoint:NSMakePoint(20,20)];
    [window setReleasedWhenClosed : YES];
    [window setTitle:@"test"];
    [window makeKeyAndOrderFront:nil];

    int count = 0;
    
                   std::thread thread([&window](){
                        std::this_thread::sleep_for(1s);

              [CATransaction begin];
                    auto native_window = (NSWindow* ) window;
            
                    auto device = MTLCreateSystemDefaultDevice();

                    auto swap_chain = [CAMetalLayer layer];
                    swap_chain.device = device;
                    swap_chain.pixelFormat = MTLPixelFormatBGRA8Unorm_sRGB;
                    swap_chain.framebufferOnly = YES;
                    swap_chain.frame = native_window.frame;
                    native_window.contentView.wantsLayer = YES;
                    swap_chain.presentsWithTransaction = YES;
                    native_window.contentView.layer = swap_chain;
                    native_window.opaque = YES;
                    native_window.backgroundColor = nil;
                    

                    auto command_queue = [device newCommandQueue];

                    auto render_pass_descriptor = [MTLRenderPassDescriptor new];

                    auto render_pass_color_attachment_descriptor = render_pass_descriptor.colorAttachments[0];
                    render_pass_color_attachment_descriptor.texture = nil;
                    render_pass_color_attachment_descriptor.loadAction = MTLLoadActionClear;
                    render_pass_color_attachment_descriptor.clearColor = MTLClearColorMake(0.2, 0.58, 0.7, 1.0);
                    render_pass_color_attachment_descriptor.storeAction = MTLStoreActionStore;
    [CATransaction commit];
                    for(;;) {
           [CATransaction begin];
                    auto render_surface = [swap_chain nextDrawable];
                    auto render_target = render_surface.texture;

                    render_pass_color_attachment_descriptor.texture = render_target;

                    auto command_buffer = [command_queue commandBuffer];
                    auto command_buffer_encoder = [command_buffer renderCommandEncoderWithDescriptor:render_pass_descriptor];
                    [command_buffer_encoder endEncoding];

                    //[command_buffer presentDrawable:render_surface];
                    [command_buffer commit];
           [command_buffer waitUntilScheduled];
            [render_surface present];
                       std::this_thread::sleep_for(200ms);
                       std::cout << "render" << std::endl;
                       
[CATransaction commit];
                    }
                }); 


    for (;;)
    {
        NSEvent *event = [NSApp nextEventMatchingMask:NSEventMaskAny untilDate:nil inMode:NSDefaultRunLoopMode dequeue:YES ];
        if (event != nil){
            uint32_t windowId = [event windowNumber];
            count++;

            [NSApp sendEvent:event];
        }
    }
                    thread.join();
    return 0;
}